
# shores

### What is this?

This is a collection of python scripts & a number of shell-scripts to automate calling of the latter (to make it cleaner, easier). Built upon Kristin Wahls' *findShortreadsMiRNAs.py*.

### What does it do?
It processes input samples from deep sequencing, perform required shortread-related analysis and build plots.

### Project structure:

* **/source**
    *high level source code. All the scripts in contained folders are called by these scripts except those in /process.* 
    * **/generate**  
        *scripts used to generate datasets from user-provided sample sets ( sampleSet -> dataset​ )* 
    * **/process**  
        *scripts to process datasets ( dataset -> dataset )*
    * **/plot**  
        *scripts used to produce analysis ( dataset -> image/file/whatever )*
* ***.sh**  
    *shellscripts to ease running of source code scripts*
* **/aux**  
    *contains various aux scripts (for example scripts to be used while preparing data for import)*  
    **NB: Should be added to PYTHON_PATH environment variable** *e.g. `export PYTHONPATH=$PYTHONPATH:/Users/r/bio/shores/aux`*
* **/genomePackages**  
    *contains genome packages - folders with all the necessary info about given genome (mmu hsa) for use of shores*
* **/sampleSets**  
    *imported datasets are placed here - these are processed by /source/generate functions to create a dataset*
* **/work**  
    *created datasets reside here to be processed by /source/process or plotted by /source/plot (resulting plots also reside here)*

### Prerequisites

* [Python 2.7, numpy, scipy, biopython, seaborne](https://docs.python.org/3.6/installing/index.html)
* [bowtie 1.1.2](http://bowtie-bio.sourceforge.net/index.shtml)
* ['bash' package from brew](http://brew.sh/)  
    *this updated version of bash should be available at /usr/local/bin/bash*  
    *modify .sh scripts to use bash from other location*
* `shores/aux` should be added to **PYTHON_PATH**

### Where to start?

Build scatter plot for provided test-sampleset:

`./formatSamples.sh hsa work/sampleUserData/ testset` - format test dataset into proper form  
`./generateDataset.sh sampleSets/testset/ scatter` - generate the dataset from */sampleSets/testset*  
`./plot.sh work/testset_scatter_*.json scatter pdf` - generate a scatter plot from the *.json* dataset  
`1`  
Done! Check work/ for .png images
