import pandas as pd
import readline
import code
import seaborn as sns


"""
include this file for some useful dbg-functions for findshortreadsmirnas.py
"""
def sayhello():
  print "hello from dbg"

def printsr(sr):
  print "    +SR+ hairpin_id: ", sr.hairpin_id,  "|mature_id: ", sr.mature_id, "|count: ", sr.count, \
  "|offset: ", sr.offset,"|sequence: ", sr.sequence, "|position: ", sr.position, \
  "|pprime: ", sr.prime, "|mat_offset: ", sr.mat_offset, "|highest_prime: ", sr.highest_prime

def printmirna(mirna):
  print "  +miRNA+ hairpin_id: ", mirna.hairpin_id, "|miRNA_id: ", mirna.miRNA_id, "|count: ", mirna.count, \
  "|offset: ", mirna.offset, "|sequence: ", mirna.sequence, "|prime: ", mirna.prime, \
  "|expressed: ", mirna.expressed, "|highest_prime: ", mirna.highest_prime, "|mature: ", mirna.mature, \
  "|hasShortreads: ", mirna.hasShortreads

def printhairpin(hairpin):
  print " +hairpin+ hairpin_id: ", hairpin.hairpin_id, "|sequence: ", hairpin.sequence, "|foldstructure: ", \
  hairpin.foldstructure, "|expressed: ", hairpin.expressed

def dumpPanda(panda, filename):
  json = panda.to_json(orient='records')
  fp = open(filename, 'w')
  fp.write(json)
  fp.close()

def readMirnaList(filename):
  li = []
  for line in open(filename):
    line = line.strip()
    spli = line.split(' ')
    sourceHairpins = [spli[0][:-3]] if len(spli) == 2 else spli[2:]
    cla, line = line.split(' ')[1], line.split(' ')[0]
    if cla == 'f':
      cla = 'fast'
    elif cla == 'o':
      cla = 'other'
    elif cla == 's':
      cla = 'slow'
    li.append({
      'id': line,
      'prime': int(line[-2:-1]),
      'class': cla,
      'sourceHairpins': sourceHairpins
    })
  return li

def fill4sutsWithZeros(data):
  fictional = []
  exp = data['exp'].unique()[0]
  prime = data['prime'].unique()[0]
  hairpinId = data['hairpinId'].unique()[0]
  for sample in range(min(data['sample'].unique()), max(data['sample'].unique())+1):
    for position in ['start', 'end']:
      if len(data[(data.position == position) & (data['sample'] == sample)].index) == 0:
        fictional.append({
            'exp': exp,
            'prime': prime,
            'hairpinId': hairpinId,
            'sample': sample,
            'prime': prime,
            'position': position,
            'rpm': 0,
            'mirnarpm': data[data['sample'] == sample]['mirnarpm'].unique()[0]
          })
  return pd.concat([data, pd.DataFrame(fictional)])

def snsSet(sizex = 30, sizey = 10, grid = True, font_scale = 1, line_width = 1):
  sns.set(font_scale=font_scale, palette='muted', rc={
     'axes.grid': grid,
     'figure.figsize':(sizex,sizey),
     'lines.linewidth':line_width,
     'font.family': 'serif',
     'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
     ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
     })

def pf(x):
  pd.set_option('display.max_rows', len(x))
  print(x)
  pd.reset_option('display.max_rows')

def i():
  vars = globals().copy()
  vars.update(locals())
  shell = code.InteractiveConsole(vars)
  shell.interact()

def setupSnsForProd(fsizeX = 30, fsizeY = 10, grid = True, font_scale = 3.5, line_width = 3):
  sns.set(font_scale=font_scale, palette='muted', rc={
   'axes.grid': grid,
   'figure.figsize':(fsizeX,fsizeY),
   'lines.linewidth':line_width,
   'font.family': 'serif',
   'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
   ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
   })
