#!/usr/local/bin/bash

# batch-format samples for OSX
# 
# example run:    
# formatSamples.sh mmu ~/samples mmu2015
# note that samples have to be 'collapsed??' and gzipped

if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]; then
  echo "usage:"
  echo "formatSamples.sh <genomePackageName> <dirWithGzippedSamples> <yourNameForSampleSet>"
  exit
fi

SAMPLES_PATH=`cd "$2"; pwd`

SRS_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

if [ ! -d "$SRS_DIR/genomePackages/$1/index/" ]; then
  echo "genomePackage $1 doesn't exist of doesn't have proper index"
  exit
fi

if [ -d "$SRS_DIR/sampleSets/$3" ]; then
  echo "$SRS_DIR/sampleSets/$3 already exists"
  echo "to not mix datasets / overwrite stuff, aborting the execution"
  echo "chose another name for your sampleSet or delete the old one"
  exit
fi

numberFastaFiles=`ls $SAMPLES_PATH/*.fasta | wc -l | xargs`
numberGZFiles=`ls $SAMPLES_PATH/*.gz | wc -l | xargs`

if [ $numberFastaFiles -ge $numberGZFiles ]; then
  if [ $numberFastaFiles -lt 1 ]; then
    echo "didn't find .fasta or .gz files, rename samples to .fasta"
    exit
  fi
  FILE_PATTERN="*.fasta"
  CAT_OP="cat"
else
  FILE_PATTERN="*.gz"
  CAT_OP="zcat"
fi

mkdir -p $SRS_DIR/sampleSets/$3
i=0
numberFiles=`ls $SAMPLES_PATH/$FILE_PATTERN | wc -l | xargs`
for file in `ls $SAMPLES_PATH/$FILE_PATTERN`
do
	i=$(($i+1))
	echo "processing $i out of $numberFiles"
  FILE_NAME=$(basename $file)
	$CAT_OP < $file | bowtie -f $SRS_DIR/genomePackages/$1/index/index - -a -v 0 -m 100 --best --norc --suppress 2,6 > $SRS_DIR/sampleSets/$3/$FILE_NAME.formatted
done

#record what genome package was used to create the sample set
echo "$1" > $SRS_DIR/sampleSets/$3/.gp

echo "		Formatted $i files (:"
