import argparse
import imp
import findShortreadsMiRNAs as fsr
import sys
import re
import gzip
import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import seaborn as sns
import os

sns.set(color_codes=True)

parser = argparse.ArgumentParser()
parser.add_argument('hairpin', help='reference hairpin.fa file')
parser.add_argument('folds', help='folds like miRNA folds')
parser.add_argument('mature', help='mature sequences properly formatted')
parser.add_argument('samples', help='sample files separated by ,')
parser.add_argument('datasetFunction', help='what kind of dataset to build? grouped_sr, dataset_stats, sr_per_offset')
parser.add_argument('output', help='output filename')

parser.add_argument('optarg1', nargs='?')
parser.add_argument('optarg2', nargs='?')
parser.add_argument('optarg3', nargs='?')
parser.add_argument('optarg4', nargs='?')
parser.add_argument('optarg5', nargs='?')

args = parser.parse_args()

class Dataset:
  def __init__(self, hairpinMap, srMap, mirnaMap):
    self.hairpinMap = hairpinMap
    self.srMap = srMap
    self.mirnaMap = mirnaMap
  @classmethod
  def fromTuple(cls, (srMap, mirnaMap, hairpinMap)):
    """
    this way you can initialize a dataset with:
    data = Dataset(findShortreadsMiRNAs(params))
    """
    return cls(hairpinMap, srMap, mirnaMap)

def main():
  # retrieve the file with user-defined dataset-building and aggregation functions:
  include_file_path = os.path.dirname(os.path.abspath(__file__)) + "/generate/" + args.datasetFunction + ".py"
  if (not os.path.isfile(include_file_path)):
    print include_file_path, " does not exists, dying"
    exit(1)
  dsGEN = imp.load_source('', include_file_path)

  samples = args.samples.split(',')
  gp_array = []

  optargs = [args.optarg1, args.optarg2, args.optarg3, args.optarg4, args.optarg5]

  #on each sample call first dataset-building function
  for sample in samples:
    if len(sample)<1:
      continue
    data = Dataset.fromTuple(fsr.main(args.hairpin, args.folds, args.mature, sample, 'xName', 'xtNAme'))
    gp_array += dsGEN.buildData(data, sample, optargs)

  #then aggregate all data into pandas dataset and write it to disk
  pdDataframe = dsGEN.buildPD(gp_array, optargs)
  json = pdDataframe.to_json(orient='records')
  fp = open(args.output, 'w')
  fp.write(json)
  fp.close()

main()
