"""
Module for building the most basic and extensive SR-dataset.
Almost directly records all data from findShortreadsMiRNAs.py
"""

import pandas as pd
import numpy as np
import code
import readline

def buildData(fsr_object, sample, args):
  filter_through = []

  # now transform first arg of form mirna-id:prime,mirna-id:prime,.... 
  # into filter-array of form [(mirnaid,prime), ...]
  # if args[0] is not None:
  #   for item in args[0].split(','):
  #     item_parts = item.split(':')
  #     filter_through.append((item_parts[0], item_parts[1]))

  arr = []
  for id in fsr_object.hairpinMap:
    if not fsr_object.hairpinMap[id].expressed:
      continue
    for srItem in fsr_object.srMap[id]:
      if len(filter_through) > 0:
        skip = True
        for filter_item in filter_through:
          if id.lower() == filter_item[0].lower() and str(srItem.prime) == filter_item[1]:
            skip = False
        if skip:
          continue
      miRNAseq, miRNAcount, the_miRNA = "na", "na", "na"
      for miRNA in fsr_object.mirnaMap[id]:
        if miRNA.prime==srItem.prime and miRNA.mature:
          miRNAseq = miRNA.sequence
          miRNAcount = miRNA.count
          the_miRNA = miRNA
      if the_miRNA == "na":
        continue
      if srItem.count > 2*miRNAcount:
        continue
      arr.append({
          'sample': sample,
          'hairpinId': fsr_object.hairpinMap[id].hairpin_id,
          'offsetHairpin': srItem.offset,
          'offsetMature': srItem.mat_offset,
          'srSequence': srItem.sequence,
          'rpm': srItem.count,
          'matureId': srItem.mature_id,
          'position': srItem.position,
          'prime': srItem.prime,
          'miRNAseq': miRNAseq,
          'miRNAcount': miRNAcount,
          'miRNAkind': 'guide' if the_miRNA.highest_prime else 'passenger'
        })
  return arr

def buildPD(builtData, args):
  frame = pd.DataFrame(builtData)
  return frame.sort_values(by=['rpm'], ascending=0) if len(builtData) > 0 else frame

