"""
Module for finding shortreads and getting some basic information about their expression. 
Used to get a sense of the level of SR-expression.
One entry per hairpin.
Was written with use on single sample in mind.
"""

import pandas as pd
import numpy as np

def buildData(fsr_object, sample, args):
  arr = []
  for id in fsr_object.hairpinMap:
    if not fsr_object.hairpinMap[id].expressed:
      continue
    if len(fsr_object.srMap[id]) > 0:
      highestSRSeq = "none"
      miRNAid = ""
      miRNAseq = ""
      highestSRCount = 0
      highestSRposition = ""
      highestSRprime = ""
      highestSRmatoffset = 0
      additionalSRs = ""
      totalCount = 0.0
      numberUqSR = 0
      numberUqSRo5 = 0
      miRNAcount = "na"
      for srItem in fsr_object.srMap[id]:
        # finding the highest-expressed sr
        totalCount += srItem.count
        if srItem.count > highestSRCount:
          highestSRCount = srItem.count
          highestSRSeq = srItem.sequence
          highestSRposition = srItem.position
          highestSRprime = srItem.prime
          highestSRmatoffset = srItem.mat_offset
          miRNAid = srItem.mature_id
      if len(fsr_object.srMap[id]) > 1:
        for srItem in fsr_object.srMap[id]:
          # statistics for the rest of shortreads
          if srItem.sequence == highestSRSeq:
            continue
          additionalSRs += srItem.sequence + ":" + "{0:.2f}".format(srItem.count) + ";"
      for miRNA in fsr_object.mirnaMap[id]:
        if miRNA.prime==highestSRprime and miRNA.mature:
          miRNAseq = miRNA.sequence
          miRNAcount = miRNA.count
      additionalSRs = additionalSRs[0:-1]
      arr.append({
          'sample': sample,
          'totalSRcount': totalCount,
          'hairpin': fsr_object.hairpinMap[id].hairpin_id,
          'highestSRSeq': highestSRSeq,
          'highestSRCount': highestSRCount,
          'highestSRprime': highestSRprime,
          'highestSRposition': highestSRposition,
          'highestSRmatoffset': highestSRmatoffset,
          'miRNAid': miRNAid,
          'miRNAseq': miRNAseq,
          'miRNAcount': miRNAcount,
          'additionalSRs': additionalSRs
        })
  return arr

def buildPD(builtData, args):
  frame = pd.DataFrame(builtData)
  return frame.sort_values(by=['totalSRcount'], ascending=0) if len(builtData) > 0 else frame

