"""
This module builds data for source/plot/groupedSR.py.
"""

import pandas as pd
import numpy as np
import pprint as p

entries = 0
maxentries = 3
def bump():
  global entries, maxentries
  entries += 1
  if entries >=maxentries:
    print "bump: reached max number of entries, gotta exit (:"
    exit()



def buildData(data, sample, args):
  """
  INPUT: Dataset data
  processes all data into a number of HairpinProduce entries. Groups them those with >10 times diff of 3'/5' rna strands expression
  then groups into hasSR?(none, only-high, only-low, both)
  """
  global allowed_offsets_for_shortreads
  # TODO unhardcode
  allowed_offsets_for_shortreads = [-1, 0, 1]
  diffT = 15.0
  notEqT = 5.0
  arr = []

  for id in data.hairpinMap:
    # raw_input("enter?")
    # auf.printHairpinData(id, data)
    if not data.hairpinMap[id].expressed:
      # print 'hairpin ', id, ' is not expressed, continuing'
      # for miRNA in data.mirnaMap[id]:
        # print 'i see ', miRNA.miRNA_id, ' ', miRNA.sequence, ' hasSR? ',miRNA.hasShortreads, ' count? ', miRNA.count
      continue
    mirna3pm, mirna5pm = 0, 0 #3-prime representative mature miRNA & 5-prime -ii-
    # print "entering loop"
    for miRNA in data.mirnaMap[id]:
      # print 'i see ', miRNA.miRNA_id, ' ', miRNA.sequence, ' hasSR? ',miRNA.hasShortreads, ' count? ', miRNA.count
      if miRNA.mature:
        if miRNA.prime == 5:
          # print '  it is 5pm, adding to mirna5pm  ', miRNA.count
          mirna5pm = max(miRNA.count, mirna5pm)
        else:
          # print '  it is 3pm setting mirna5pm  ', miRNA.count
          mirna3pm = max(miRNA.count, mirna3pm)
    # print 'mirna3pm#', mirna3pm, ' mirna5pm#', mirna5pm
    #!! nice, have miRNA numbers, time to chose the first category:
    group, high, low = None, 0, 0
    if mirna3pm == 0 or mirna3pm < 0.5:
      if mirna5pm < diffT*0.5:
        # print '!! mirna3pm under treshold, mirna5pm < diffT*treshold'
        group, high, low = 'none', 5, 3
      else:
        #!! mirna3pm == 0 && mirna.expressed => mirna5pm != 0
        # print '!! mirna3pm under treshold, mirna5pm > diffT*treshold'
        group, high, low = 'diff', 5, 3
    elif mirna5pm == 0 or mirna5pm < 0.5:
      #!! at this point: mirna3pm > 0.5
      if mirna3pm < diffT*0.5:
        # print '!! mirna5pm under treshold, mirna3pm < diffT*treshold'
        group, high, low = 'none', 3, 5
      else:
        # print '!! mirna5pm under treshold, mirna3pm > diffT*treshold'
        group, high, low = 'diff', 3, 5
    else:
      if mirna5pm / mirna3pm > diffT:
        group, high, low = 'diff', 5, 3
      elif mirna3pm / mirna5pm > diffT:
        group, high, low = 'diff', 3, 5
      elif mirna5pm / mirna3pm > notEqT:
        group, high, low = 'none', 5, 3
      elif mirna3pm / mirna5pm > notEqT:
        group, high, low = 'none', 3, 5
      else:
        group = 'eq'
        if (mirna5pm > mirna3pm):
          high, low = 5, 3
        else:
          high, low = 3, 5
    ##time to caluculate shortreads sums
    sr3pCount, sr5pCount = 0, 0
    for sr in data.srMap[id]:
      if not sr.mat_offset in allowed_offsets_for_shortreads:
        # dPrint('skipping SR since not in allowed offset range')
        continue
      # dPrint('added SR to count')
      if sr.prime == 5:
        if sr.count > 2*mirna5pm:
          continue
        sr5pCount += sr.count
      else:
        if sr.count > 2*mirna3pm:
          continue
        sr3pCount += sr.count
    # if sr3pCount == 0 and sr5pCount == 0:
      # continue
    ##deside on the second category based on sr expression
    secondaryGroup = None
    if sr5pCount == 0 and sr3pCount == 0:
      secondaryGroup = 'none'
    elif (sr5pCount == 0 and high == 5) or (sr3pCount == 0 and high == 3):
      secondaryGroup = 'low'
    elif sr5pCount == 0 or sr3pCount == 0:
      secondaryGroup = 'high'
    else:
      secondaryGroup = 'both'
    arr.append({
      'hairpin_id': id,
      '3primemiRNAcount': mirna3pm,
      '5primemiRNAcount': mirna5pm,
      '3primeSRcount': sr3pCount,
      '5primeSRcount': sr5pCount,
      'group': group,
      'subgroup': secondaryGroup,
      'sample': sample
      })
  return arr

def buildPD(builtData, args):
  frame = pd.DataFrame(builtData)
  # return frame.sort_values(by=['srcount'], ascending=0) if len(builtData) > 0 else frame
  return frame

