"""
This module builds data for source/plot/scatter.py - basic scatter plot on miRNA-sr expression.
"""

import pandas as pd
import numpy as np
import pprint as p
import readline
import code

allowed_mat_offset = [-1, 0, 1]
entries = 0
maxentries = 3
def bump():
  global entries, maxentries
  entries += 1
  if entries >=maxentries:
    print "bump: reached max number of entries, gotta exit (:"
    exit()
start_end_threshold = 10

def buildData(fsr_object, sample, args):
  global mat_offset
  global start_end_threshold

  # construct the pre-resultobject
  arr = []
  for id in fsr_object.hairpinMap:
    if not fsr_object.hairpinMap[id].expressed:
      continue
    for miRNAItem in fsr_object.mirnaMap[id]:
      if not miRNAItem.hasShortreads:
        continue
      for srItem in fsr_object.srMap[id]:
        if not srItem.mat_offset in allowed_mat_offset:
          continue
        if miRNAItem.miRNA_id != srItem.mature_id:
          #sr sr belongs to another miRNA
          continue
        rpm = srItem.count
        if srItem.count > miRNAItem.count*2:
          #fake sr
          continue
        if rpm > 0:
          arr.append({
            'sample': sample,
            'hairpinId': fsr_object.hairpinMap[id].hairpin_id,
            'matureId': miRNAItem.miRNA_id,
            # 'prime': miRNAItem.prime,
            # 'highest_prime': miRNAItem.highest_prime,
            'miRNAcount': miRNAItem.count,
            'rpm': rpm,
            'position': srItem.position,
            'temp_srSequence': srItem.sequence,
            'temp_miRNAseq': miRNAItem.sequence,
            })

  data = pd.DataFrame(arr)

  # read original file of the sample and get all counts
  # needed for raw_count statistics
  the_counts = {}
  for line in open(sample):
    s = line.split('\t')
    seq = s[3]
    reads = int(s[0].split('-')[1])
    the_counts[seq] = reads
  data['rawSRcount'] = data.apply(lambda e: the_counts[e['temp_srSequence']], axis = 1)
  data['rawMIRNAcount'] = data.apply(lambda e: the_counts[e['temp_miRNAseq']], axis = 1)

  # compile pre-resultobject into resultobject
  dict = []
  datagroupBysampleByhairpin = data.groupby('hairpinId')
  for hairpin, datagroupX in datagroupBysampleByhairpin:
    datagroupBysampleByhairpinByPos = datagroupX.groupby('position')
    for position, datagroupXX in datagroupBysampleByhairpinByPos:
      rawMIRNAcount = countRmiRNAs(datagroupXX, 'rawMIRNAcount')
      miRNAcount = countRmiRNAs(datagroupXX, 'miRNAcount')
      dict.append({
        'hairpinId': hairpin,
        'sample': sample,
        'position': position,
        'miRNAcount': miRNAcount,
        'rpm': datagroupXX['rpm'].sum(),
        'rawSRcount': datagroupXX['rawSRcount'].sum(),
        'rawMIRNAcount': rawMIRNAcount
        })

  return dict

def buildPD(builtData, args):
  frame = pd.DataFrame(builtData)
  return frame.sort_values(by=['rpm'], ascending=0) if len(builtData) > 0 else frame

def countRmiRNAs(data, field):
  counter = 0
  for matureId, group in data.groupby('matureId'):
    counter += group[field].unique()[0]
  return counter

