"""
takes as input 1_*_processed_fastslow_sr_stats.json type dataset
and builds 4 different statistics on it. 

pipeline: 
  ./generate * 1;
  source/process/4suprocess.py *;
  source/process/4su_fastslow_correlation.py *;
  <this> *
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import readline
import code
import shorescommon

output_format = "png"


def plot(data, output, args):
  global output_format
  if args[0] == 'pdf':
    output_format = "pdf"

  plotSrBox(data, output.replace('.png', '_srXmirnaCorrBoxPlot.png')) #correlation values for fast/slow/other by start/end
  plotRpmPerClass(data, output.replace('.png', '_srXmirnaRpmBoxPlot.png')) #sr.rpm / mirna.rpm values boxplot for fast/slow/oth
  plotMeanRpm(data, output.replace('.png', '_meanRpmBarChart.png')) # simple mean rpm value barcharts for all classes
  plotAbsoluteRpmValues(data, output.replace('.png', '_absRpmValsBoxPlot.png')) # boxplot of rmp values of sr/mirna

def plotAbsoluteRpmValues(data, output):
  global output_format
  if output_format == "pdf":
    shorescommon.setupSnsForProd(30,20)
    output = output.replace('.png', '.pdf')
  else:
    shorescommon.snsSet(16, 14)

  f, ((ax_sr, ax_sr_log), (ax_mirna, ax_mirna_log)) = plt.subplots(2,2)
  # f, (ax_sr, ax_mirna, ax_sr_mirna_log) = plt.subplots(1,3)

  d = data.groupby(['mirna_id', 'position']).agg({
    'sr_rpm': lambda e: e.mean(),
    'mirna_rpm': lambda e: e.mean(),
    'class': lambda e: e.iloc[0]
    }).reset_index()

  bpargs = {
    'hue': 'class',
    'order': ['end', 'start'],
    'hue_order': ['fast', 'slow', 'other'],
    'showfliers': False
  }
  sns.boxplot(data=d, x = 'position', y = 'sr_rpm', ax=ax_sr, **bpargs)
  sns.boxplot(data=d, x = 'position', y = 'sr_rpm', ax=ax_sr_log, **bpargs)
  ax_sr_log.set_yscale('log')
  for ax in [ax_sr_log, ax_sr]:
    ax.set_xlabel("")
    ax.set_ylabel("RPM(sr)")
  ax_sr.set_title('SR expression')
  ax_sr_log.set_title('SR expression [log]')


  d_ = d.copy()
  d_['position'] = d_.apply(lambda e: 'miRNA has end-sr' if e['position'] == 'end' else 'miRNA has start-sr', axis = 1)

  bpargs['order'] = ['miRNA has end-sr', 'miRNA has start-sr']
  sns.boxplot(data=d_, x = 'position', y = 'mirna_rpm', ax=ax_mirna, **bpargs)
  sns.boxplot(data=d_, x = 'position', y = 'mirna_rpm', ax=ax_mirna_log, **bpargs)
  ax_mirna_log.set_yscale('log')
  for ax in [ax_mirna, ax_mirna_log]:
    ax.set_xlabel('')
    ax.set_ylabel('RPM(miRNA)')
  ax_mirna.set_title('miRNA expression')
  ax_mirna_log.set_title('miRNA expression [log]')


  plt.tight_layout()
  plt.savefig(output)
  plt.close()
  print output

def plotMeanRpm(data, output):
  global output_format
  if output_format == "pdf":
    shorescommon.setupSnsForProd(30,10)
    output = output.replace('.png', '.pdf')
  else:
    shorescommon.snsSet(16, 7)

  f, (ax2,ax3) = plt.subplots(1,2)

  d_mirna = data.groupby(['class', 'position']).agg({
    'mirna_rpm': lambda e: e.sum(), 
    'mirna_id': lambda e: e.count()
  })
  d_mirna['rpm'] = d_mirna['mirna_rpm'] / d_mirna['mirna_id']

  d_sr = data.groupby(['class', 'position']).agg({
    'sr_rpm': lambda e: e.sum(), 
    'mirna_id': lambda e: e.count()
  })
  d_sr['rpm'] = d_sr['sr_rpm'] / d_sr['mirna_id']
  d_sr['kind'], d_mirna['kind'] = 'sr', 'mirna'

  # d['hue'] = d['kind'] + '-' + d['position']
  d_sr.reset_index(inplace = True)
  d_mirna.reset_index(inplace = True)
  d_sr['hue'] = d_sr['position']
  d_mirna['hue'] = d_mirna.apply(lambda e: 'miRNA has ' + str(e['position']) + '-sr', axis = 1)

  bpargs = {
    'hue': 'class',
    'hue_order': ['fast', 'slow', 'other']
  }
  
  bpargs['order'] = ['end', 'start']
  sns.barplot(x='hue', y='rpm', data=d_sr, ax=ax2, **bpargs)
  bpargs['order'] = ['miRNA has end-sr', 'miRNA has start-sr']
  sns.barplot(x='hue', y='rpm', data=d_mirna, ax=ax3, **bpargs)

  for ax in [ax2, ax3]:
    ax.set_xlabel("")
    ax.set_ylabel('mean rpm')
  
  ax2.set_title('Mean expression values (shortreads)')
  ax3.set_title('Mean expression values (miRNA)')


  plt.tight_layout()
  plt.savefig(output)
  plt.close()
  print output

def plotRpmPerClass(data, output):
  global output_format
  if output_format == "pdf":
    shorescommon.setupSnsForProd(30,10)
    output = output.replace('.png', '.pdf')
  else:
    shorescommon.snsSet(16, 7)

  data['sr/mirnarpm'] = data['sr_rpm']/data['mirna_rpm']

  f, (ax1,ax2) = plt.subplots(1,2)
  ax2.set_yscale('log')

  bpargs = {
    'hue': 'position',
    'order': ['fast', 'other', 'slow'],
    'hue_order': ['end', 'start'],
    'showfliers': False
  }
  # ax1.set_title('Unmodified Pearson');ax2.set_title('Unmodified Spearman');ax3.set_title('Experiment-mean Pearson');ax4.set_title('Experiment-mean Spearman')
  ax1.set_title('SR/miRNA expression relationship')
  ax2.set_title('SR/miRNA expression relationship [log]')

  d = data.groupby(['mirna_id', 'position']).agg({
    'sr/mirnarpm': lambda e: e.mean(), 
    'class': lambda e: e.iloc[0],
    'position': lambda e: e.iloc[0]
    })
  sns.boxplot(x='class', y='sr/mirnarpm', data=d, ax=ax1, **bpargs)
  sns.boxplot(x='class', y='sr/mirnarpm', data=d, ax=ax2, **bpargs)

  for ax in [ax1, ax2]:
    ax.set_ylabel("")
    ax.set_xlabel("")
  ax1.set_ylabel("SR.rpm / miRNA.rpm")

  plt.tight_layout()
  plt.savefig(output)
  plt.close()
  print output


def plotSrBox(data, output):
  """for each hairpinId take mean across samples"""
  global output_format
  if output_format == "pdf":
    shorescommon.setupSnsForProd(30,20)
    output = output.replace('.png', '.pdf')
  else:
    shorescommon.snsSet(16, 14)

  d = data.groupby(['mirna_id', 'position']).agg({
    'corr_p': lambda e: e.mean(), 
    'corr_s': lambda e: e.mean(), 
    'class': lambda e: e.iloc[0],
    'position': lambda e: e.iloc[0]
    })
  f, ((ax1,ax3), (ax2, ax4)) = plt.subplots(2,2, sharey=True)

  ax1.set_title('Unmodified Pearson');ax2.set_title('Unmodified Spearman');ax3.set_title('Experiment-mean Pearson');ax4.set_title('Experiment-mean Spearman')
  bpargs = {
    'hue': 'position',
    'order': ['fast', 'other', 'slow'],
    'hue_order': ['end', 'start'],
    'showfliers': False
  }
  sns.boxplot(x='class', y='corr_p', data=data, ax=ax1, **bpargs)
  sns.boxplot(x='class', y='corr_s', data=data, ax=ax2, **bpargs)
  sns.boxplot(x='class', y='corr_p', data=d, ax=ax3, **bpargs)
  sns.boxplot(x='class', y='corr_s', data=d, ax=ax4, **bpargs)
  ax1.set_ylabel('Correlation value')
  ax1.set_xlabel('')
  for ax in [ax2, ax3, ax4]:
    ax.set_ylabel("")
    ax.set_xlabel("")
  plt.tight_layout()
  plt.savefig(output)
  plt.close()
  print output




