"""
takes as input 1_*_processed.json type dataset and generates timeseries sr-to-mirna levels
for sr<->miRNA pairs that have enough data # (todo: explain)

This is not a comprehensive plot, it just displays some of the more prominent cases.

pipeline:
  ./generate * 1;
  source/process/4suprocess.py *;
  <this> *
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import shorescommon
import code
import readline

c_experiments = ['EXP1', 'EXP2', 'EXP3']
output_format = 'png'

def screenMirnaList(mirnaList, data):
  accumulator = []
  for i, mirna in enumerate(mirnaList):
    idata = data[(data.hairpinId == mirna['sourceHairpins'][0]) & (data.prime == mirna['prime'])]
    num_full_experiments = 0
    for x, exp in enumerate(c_experiments):
      iidata = idata[idata['exp'] == exp]
      if (iidata.groupby('sample').__len__() == 7):
        num_full_experiments += 1
    if (num_full_experiments > 0):
      accumulator.append({
          'id': mirna['sourceHairpins'][0],
          'prime': mirna['prime'],
          'size': idata.groupby('sample').__len__(),
          'num_full_experiments': num_full_experiments,
          'class': mirna['class']
        })
  return sorted(accumulator, key=lambda x: x['num_full_experiments'], reverse=True)

def plot(data,output,args):
  global output_format
  if args[0] == 'pdf':
    output_format = "pdf"

  mirnaList = shorescommon.readMirnaList('work/miRNAlists/4sumirnas.txt')

  #remove confusing mirnas with multiple hairpin sources (should ideally be implemented without removing them in the future)
  mirnaList = filter(lambda e: len(e['sourceHairpins']) == 1, mirnaList)

  mirnaList = screenMirnaList(mirnaList, data)

  #leave only those with data on each of three experiments
  mirnaList = filter(lambda e: e['num_full_experiments'] == 3, mirnaList)

  tsPlotSelectedMirnas(data, enumerateOutputName(output, '_fast'), filter(lambda e: e['class'] == 'fast', mirnaList))
  tsPlotSelectedMirnas(data, enumerateOutputName(output, '_slow'), filter(lambda e: e['class'] == 'slow', mirnaList))
  tsPlotSelectedMirnas(data, enumerateOutputName(output, '_slow'), filter(lambda e: e['class'] == 'other', mirnaList))


def placeTsPlotOnAxis(data, ax, label):
  data = data.sort_values(by='position')
  if (data.groupby('sample').__len__() == 7):
    mirnaplotting_data = data.groupby('sample').first().reset_index()
    mirnaplotting_data['position'] = 'start'
    sns.tsplot(mirnaplotting_data, time='sample', unit='hairpinId', condition='position', value='mirnarpm', ax=ax, color='red', legend=False, interpolate=True)
    ax2 = ax.twinx()
    ax.set_ylabel("RPM(miRNA)")
    if len(data.index) < 14:
      data = shorescommon.fill4sutsWithZeros(data)
    data.sort_values(by='sample')
    sns.tsplot(data, time='sample', unit='hairpinId', condition='position', value='rpm', ax = ax2, legend=True, interpolate=True, marker='.')
    ax2.set_ylabel("RPM(sr)")
    # ax2.set_yticks([])
    ax2.grid(b=False)
    left, width = .25, .4
    bottom, height = .25, .7
    right = left + width
    top = bottom + height
    ends = data[data.position == 'end']
    starts = data[data.position == 'start']
    ax.text(right+0.2, top+0.1, label + '; end:%.2f' % ends['rpm'].corr(ends['mirnarpm']) + ' start:%.2f' % starts['rpm'].corr(starts['mirnarpm']),
        horizontalalignment='right',
        verticalalignment='top',
        transform=ax.transAxes)


def tsPlotSelectedMirnas(data, output, mirnaList):
  global output_format

  item_per_file = 4
  num_rows = len(mirnaList)
  num_files = num_rows / item_per_file + 1

  for file_number in range(num_files):
    current_now_rows = min(item_per_file, num_rows - file_number * item_per_file)

    if output_format == "pdf":
      shorescommon.setupSnsForProd(30,15, True, 1.6, 1)
      output = output.replace('.png', '.pdf')
    else:
      shorescommon.snsSet(16, 8, True, 0.8, 1)

    # sns.set(font_scale=0.8, palette='muted', rc={
    #  'axes.grid': True,
    #  'figure.figsize':(15,4*2),
    #  'lines.linewidth':1,
    #  'legend.frameon': False,
    #  'font.family': 'serif',
    #  'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
    #  ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
    #  })
    f, (axes) = plt.subplots(4, 3)

    for y in range(current_now_rows):
      explored_mirna = mirnaList[file_number*item_per_file + y]
      for x, experiment in enumerate(c_experiments):
        ax = axes[y][x]
        placeTsPlotOnAxis(data[(data['hairpinId'] == explored_mirna['id']) & (data['exp'] == experiment) \
            & (data['prime'] == explored_mirna['prime'])], \
          ax, experiment + " " + explored_mirna['id']+" "+str(explored_mirna['prime'])+'p')

    plt.tight_layout()
    plt.savefig(enumerateOutputName(output, '_'+str(file_number)))
    plt.close()
    print enumerateOutputName(output, '_'+str(file_number))

def enumerateOutputName(name, id):
  return name[0:name.rfind('.')] + id + name[name.rfind('.'):]

def annotate(plt, sampleName):
  plt.suptitle('sample: ' + '_' + sampleName, fontsize=12)