"""
Demo-plot for demonstration of lines of discrete sr-mirna relationships caused by low raw reads.


pipeline: 
  ./generate * scatter;
  ./generate * 1;
  source/process/inscribeOriginalCountsTo1.py <dataset>_1_*.json;
  ./plot.sh <dataset>_scatter_*.json scatterOC <dataset>_1_*_withOC.json
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import code
import readline
import sys
import shorescommon

disc_values = [0.33333, 0.5, 1, 1.33333, 2, 3]
marker = 'o'
ocdata = None

def croc(data):
  global ocdata
  for i, entry in data.iterrows():
    explored_rel = data['rel'].unique()[0]
    # hsa-mir-6087 & mirna & 1 & 2 & 0.2818333598
    current_hairpinID = entry['hairpinId']
    printMi = True
    for j, croced_entry in \
    ocdata[(ocdata['sample'] == entry['sample']) & (ocdata['hairpinId'] == entry['hairpinId'])].iterrows():
      if round(croced_entry['rpm']/croced_entry['miRNAcount'], 5) != explored_rel:
        continue
      if printMi:
        print current_hairpinID, " & mirna & ", croced_entry['rawMIRNAalignments'], " & ", croced_entry["rawMIRNAcount"],\
          " & ", croced_entry['miRNAcount']
        printMi = False
      print current_hairpinID, " & shortread & ", croced_entry['rawSRalignments'], " & ", croced_entry['rawSRcount'],\
        " & ", croced_entry['rpm'], "\t"
def plot(data, output, args):
  global ocdata

  output = output.replace('.png', '.pdf')

  ocdata = pd.read_json(open(args[0], 'r'))

  data['rel'] = data.apply(lambda e: e['rpm'] / e['miRNAcount'], axis = 1)
  data['rel'] = data.apply(lambda e: round(e['rel'], 5), axis = 1)
  data['rawCsum'] = data['rawSRcount'] + data['rawMIRNAcount']
  dda = []
  for valu in disc_values:
    if len(data[data.rel == valu].index) > 0:
      dda.append(data[data.rel == valu])
      data = data[data.rel != valu]

  plotR(data)


  plt.gca().set_title('Scatter plot: sr-to-miRNA expression')

  plt.tight_layout()
  plt.savefig(output.replace('.pdf', '_1.pdf'))
  plt.close()



  plotR(data[data.rawCsum <= 10])

  plotA(data[(data.rawCsum > 10) & (data.rawCsum <=100)], "raw reads > 10")
  plotA(data[(data.rawCsum > 100)  & (data.rawCsum <=1000)], "raw reads > 100")
  plotA(data[data.rawCsum > 1000], "raw reads > 1000")

  plt.gca().set_title('Scatter plot: sr-to-miRNA expression')

  plt.tight_layout()
  plt.savefig(output.replace('.pdf', '_3.pdf'))
  plt.close()




  plt.close()
  plotR(data)
  plotAA(dda)

  plt.gca().set_title('Scatter plot: discrete sr/miRNA values')

  plt.tight_layout()
  plt.savefig(output.replace('.pdf', '_2.pdf'))
  plt.close()


def plotAA(array_with_data):
  for i, e in enumerate(array_with_data):
    plotA(e, "SR/miRNA == " + str(e['rel'].unique()[0]))

def checkSampleNames(samplenames):
  """
  checks possibility to use them as suffixes for plots - must be unique and <16 symbols long 
  """
  retval = True
  for sample in samplenames:
    if len(sample) > 16 or len(sample) < 1:
      retval = False
      break
    h = 0
    for sampleY in samplenames:
      if sampleY == sample:
        h += 1
    if h > 1:
      retval = False
      break
  return retval

  #plotReference
def pplotR():
  global marker
  data = pd.DataFrame()
  sns.set(font_scale=1, palette='muted', rc={
   'axes.grid': True,
   'figure.figsize':(13,13),
   'lines.linewidth':1,
   'font.family': 'serif',
   'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
   ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
  })
  ax = plt.gca()
  ax.set_yscale('log')
  ax.set_xscale('log')
  sns.regplot(x="miRNAcount", y="rpm", data=data, label='reference', marker=marker, fit_reg=False)

def plotR(data, label = 'reference'):
  # sns.set(font_scale=1, palette='muted', rc={
  #  'axes.grid': True,
  #  'figure.figsize':(13,13),
  #  'lines.linewidth':1,
  #  'font.family': 'serif',
  #  'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
  #  ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
  # })
  shorescommon.setupSnsForProd(20,20, True, 2.8, 2.6)
  ax = plt.gca()
  ax.set_yscale('log')
  ax.set_xscale('log')
  sns.regplot(x="miRNAcount", y="rpm", data=data, label=label, marker=marker, fit_reg=False)

  #plotActualData
def position_(data):
  def hasBothPositions(e, data):
    if len(data[(data['sample'] == e['sample']) & (data.hairpinId == e.hairpinId)].index) == 2:
      return 'both'
    else:
      return e['position']
  data['position_'] = data.apply(lambda e: hasBothPositions(e, data), axis = 1)
  return data
def plotA(data, label):
  axx = sns.regplot(x="miRNAcount", y="rpm", data=data, label=label, marker=marker, fit_reg=False)
  handles, labels = axx.get_legend_handles_labels()
  data = {l: h for h, l in zip(handles, labels)}
  legend_data = data
  # label_order = ['reference', 'actual']
  blank_handle = mpl.patches.Patch(alpha=0, linewidth=0)
  handles = [legend_data.get(l, blank_handle) for l in labels]
  plt.gcf().legends = []
  figlegend = plt.gcf().legend(handles, labels, "best", {})
  # plt.tight_layout()




