"""

pipeline: 
  ./generate * 1;
  ./plot.sh <dataset>_1_*.json amount_sr_barplot pdf
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import readline
import code
import shorescommon

def plot(data, output, args):
  if args[0] == 'pdf':
    shorescommon.setupSnsForProd(30,20)
    output = output.replace('.png', '.pdf')
  else:
    shorescommon.snsSet(16, 14)

  f, (ax1,ax2) = plt.subplots(2,1)

  data['offset'] = data.apply(lambda e: str(e['offsetMature']) if e['offsetMature'] in [-1,0,1] else 'other', axis = 1)
  data['kind'] = data.apply(lambda e: e['miRNAkind'] + ' ' + e['position'], axis = 1)

  
  xdata = data.copy()
  xdata = xdata.groupby(['kind', 'offset']).agg({
    'rpm': {'count(rpm)': lambda e: e.count()}
  })
  xdata.columns = xdata.columns.droplevel(0)
  sns.barplot(data=xdata.reset_index(), x = 'offset', y = 'count(rpm)', hue='kind', order=['-1', '0', '1', 'other'], hue_order=['guide start', 'guide end', 'passenger start', 'passenger end'], ax=ax1)

  xdata = data.copy()
  xdata = xdata.groupby(['kind', 'offset']).agg({
    'rpm': {'sum(rpm)': lambda e: e.sum()}
  })
  xdata.columns = xdata.columns.droplevel(0)
  sns.barplot(data=xdata.reset_index(), x = 'offset', y = 'sum(rpm)', hue='kind', order=['-1', '0', '1', 'other'], hue_order=['guide start', 'guide end', 'passenger start', 'passenger end'], ax=ax2)
  

  for ax in [ax1, ax2]:
    ax.set_xlabel("Offset")
  ax1.set_ylabel("Count")
  ax2.set_ylabel("Total RPM")
  ax1.set_title("Shortread distribution by offset and strand (guide/passenger)")

  plt.tight_layout()
  plt.savefig(output)
  plt.close()
  print output



