"""
To use this plot:
run generate/groupedSR.py
run aux/gradeExpression.py on the generated dataset
plug the result into this script

  ./plot.sh work/srds_groupedSR_*_graded_*.json groupedSR pdf
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import code
import readline
import pprint
import shorescommon

percentile_threshold = 60

def plot(data, output, args):
  data = transformData(data)

  if args[0] == 'pdf':
    shorescommon.setupSnsForProd(30,20, True, 3)
    output = output.replace('.png', '.pdf')
  else:
    sns.set(font_scale=1, palette='muted', rc={
     'axes.grid': True,
     'figure.figsize':(25,12),
     'lines.linewidth':1,
     'font.family': 'serif',
     'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
     ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
     })

  f, (ax1, ax2) = plt.subplots(2)

  maxGrade = data['expressionGrade'].unique()[-1]
  data['percentile'] = data.apply(lambda e: \
    int(float(e['expressionGrade']) / (maxGrade + 1) * 100), \
    axis = 1)

  global percentile_threshold
  for percentile_value in data['percentile'].unique():
    if percentile_value >= percentile_threshold:
      percentile_threshold = percentile_value
      break

  data['percentile'] = data.apply(lambda e: \
    str(e['percentile']) + 'th percentile' if e['percentile'] > percentile_threshold \
    else str(percentile_threshold) + 'th percentile', \
    axis = 1)

  sns.boxplot(x='group', y='#subgroup/#total', data=data, ax=ax1, hue='percentile', order=['diff.high', 'diff.both', 'diff.low', 'eq.high', 'eq.both', 'eq.low'])
  sns.boxplot(x='percentile', y='#subgroup/#total', data=data, ax=ax2, hue='group', hue_order=['diff.high', 'diff.both', 'diff.low', 'eq.high', 'eq.both', 'eq.low'])
  ax2.set_xlabel('')


  if args[1] == 'hack1':
    ax1.set_title('4SU: Distribution of SR-presence per miRNA grouped by relative expression of miRNA strands')

  plt.tight_layout()
  plt.savefig(output)

def transformData(data):

  # removing fake SR? 
    # no longer needed as groupedSR generator cuts off 2xmiRNA shortreads
  # originalDataLen = len(data.index)
  # discarded1 = data[data['3primeSRcount'] > 2*data['3primemiRNAcount']]
  # discarded2 = data[data['5primeSRcount'] > 2*data['5primemiRNAcount']]
  # discarded_pd = pd.concat([discarded1, discarded2])
  # data = data[data['3primeSRcount'] <= 2*data['3primemiRNAcount']]
  # data = data[data['5primeSRcount'] <= 2*data['5primemiRNAcount']]
  # print originalDataLen - len(data.index), ' out of ', originalDataLen, ' entries were discarded'

  arr = []
  data['expression'] = data['5primemiRNAcount'] + data['3primemiRNAcount']

  def expressionlevel_separator(expression):
    if expression > sepValues[0]:
      if expression > sepValues[1]:
        return str(sepValues[1]) + '-...'
      else:
        return str(sepValues[0]) + '-' + str(sepValues[1])
    else:
      return '0-'+str(sepValues[0])

  groupexpression = data.groupby('expressionGrade')

  for expressionGrade, datagroup in groupexpression:
    groupby = datagroup.groupby('sample')
    for item, group in groupby:
      srgroupby = group.groupby(['group', 'subgroup'])
      total = {}
      for srgpitem, srgpgroup in srgroupby:
        if srgpitem[0] in total:
          total[srgpitem[0]] += len(srgpgroup)
        else:
          total[srgpitem[0]] = len(srgpgroup)
        # print srgpitem[0], srgpitem[1]
      for srgpitem, srgpgroup in srgroupby:
        # if srgpitem[1] == 'none' or srgpitem[0] == 'none':
          # continue
        arr.append({
            'group': srgpitem[0]+'.'+srgpitem[1],
            '#subgroup/#total': len(srgpgroup)*1.0/total[srgpitem[0]],
            'expressionGrade': expressionGrade,
            'sample': item
          })
  return pd.DataFrame(arr)