"""
scatter-plot for shortreads

pipeline: 
  ./generate * scatter;
  ./plot.sh <dataset>_scatter_*.json scatter pdf
"""
import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import code
import readline
import shorescommon
import sys

output_format = 'pdf'
master_title = None

def plot(data, output, args):

  global output_format, master_title
  if args[0] == 'pdf':
    output_format = "pdf"
    output = output.replace('.png', '.pdf')
  if args[1] == 'hack1':
    master_title = "Shortreads-to-miRNA expression in 4SU samples"

  data['rawCsum'] = data['rawSRcount'] + data['rawMIRNAcount']
  data = data[data['rawCsum'] > 10]
  # data = data[data['rawSRcount'] > 5 ]
  if (len(data.groupby('sample')) > 1):
    print 'data consists for multiple samples. \nEnter \'1\' if you want them to be plotted on one image, \'2\' for image-per-sample.'
    i = sys.stdin.read(1)
    # i = '2'
    if i == '1':
      plotToFile(data, output)
    elif i == '2':
      c = 0
      samplenames = []
      # next block checks whether i can just plug samplename into output name
      for sample, h in data.groupby('sample'):
        samplenames.append(normalizeSampleName(sample))
      useSampleNames = checkSampleNames(samplenames)

      for sample, datagroup in data.groupby('sample'):
        c += 1
        plotname = normalizeSampleName(sample) if useSampleNames else str(c)
        plotToFile(datagroup, enumerateOutputName(output, plotname), sample)
    else:
      print 'did not understand command, exiting'



def checkSampleNames(samplenames):
  """
  checks possibility to use them as suffixes for plots - must be unique and <16 symbols long 
  """
  retval = True
  for sample in samplenames:
    if len(sample) > 16 or len(sample) < 1:
      retval = False
      break
    h = 0
    for sampleY in samplenames:
      if sampleY == sample:
        h += 1
    if h > 1:
      retval = False
      break
  return retval

def plotToFile(data, output, sample_name=None):
  sample_name = normalizeSampleName(sample_name) if sample_name is not None else ""

  global output_format, master_title
  additional_kws = {}
  if output_format == 'png':
    sns.set(font_scale=1, palette='muted', rc={
     'axes.grid': True,
     'figure.figsize':(13,13),
     'lines.linewidth':1,
     'font.family': 'serif',
     'font.serif': """Computer Modern Roman, Bitstream Vera Serif, New Century Schoolbook, Century Schoolbook L, Utopia,\
     ITC Bookman, Bookman, Nimbus Roman No9 L, Times New Roman, Times, Palatino, Charter, serif"""
    })
  elif output_format == 'pdf':
    additional_kws = {'scatter_kws': {"s": 150},
    'marker': 'o'}
    shorescommon.setupSnsForProd(30,30)

  ax = plt.gca()
  ax.set_yscale('log')
  ax.set_xscale('log')

  sns.regplot(x="miRNAcount", y="rpm", data=data[data['position'] == 'end'], label='end', fit_reg=False, **additional_kws)
  axx = sns.regplot(x="miRNAcount", y="rpm", data=data[data['position'] == 'start'], label='start', fit_reg=False, **additional_kws)

  axx.set_ylabel('Exp(SR)')
  axx.set_xlabel('Exp(MiRNA)')
  if master_title is not None:
    axx.set_title(master_title)
  else:
    axx.set_title('Shortreads-to-miRNA expression in sample')

  handles, labels = axx.get_legend_handles_labels()
  data = {l: h for h, l in zip(handles, labels)}

  legend_data = data
  label_order = ['start', 'end']
  blank_handle = mpl.patches.Patch(alpha=0, linewidth=5)
  handles = [legend_data.get(l, blank_handle) for l in label_order]
  figlegend = plt.gcf().legend(handles, label_order, "center right", {})

  plt.tight_layout()
  print output
  plt.savefig(output)
  plt.close()

def normalizeSampleName(sample_name):
  sample_name = sample_name[sample_name.rfind('/')+1:]
  return sample_name[0:sample_name.find('.')]  

def enumerateOutputName(name, id):
  return name[0:name.rfind('.')] + id + name[name.rfind('.'):]

def annotate(plt, sampleName):
  plt.suptitle('sample: ' + '_' + sampleName, fontsize=12)

