"""
for 4SU dataset. To be run after source/process/4suprocess.py
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import shorescommon
import code
import argparse
import readline
import sys

c_experiments = ['EXP1', 'EXP2', 'EXP3']
minimum_entries = 5

parser = argparse.ArgumentParser()
parser.add_argument('dataset', help='dataset file')

args = parser.parse_args()

def filterDataToMirna(data, mirna):
  res_data = None
  return data[data.apply(lambda e: e['hairpinId'] in mirna['sourceHairpins'] and e['prime'] == mirna['prime'], axis = 1)]

def screenMirnaList(mirnaList, data):
  accumulator = []
  for i, mirna in enumerate(mirnaList):
    print '.',
    sys.stdout.flush()
    idata = filterDataToMirna(data, mirna)
    num_full_experiments = 0
    for x, exp in enumerate(c_experiments):
      iidata = idata[idata['exp'] == exp]
      if (iidata.groupby('sample').__len__() == 7):
        num_full_experiments += 1
    # print num_full_experiments, mirna
    if (num_full_experiments > 0):
      accumulator.append({
          'id': mirna['id'],
          'class': mirna['class'],
          'sourceHairpins': mirna['sourceHairpins'],
          'prime': mirna['prime'],
          'size': idata.groupby('sample').__len__(),
          'num_full_experiments': num_full_experiments
        })
  print ''
  return sorted(accumulator, key=lambda x: x['num_full_experiments'], reverse=True)

def main():
  data = pd.read_json(open(args.dataset, 'r'))
  output_dataset = args.dataset.replace('.json', '_fastslow_sr_stats.json')
  mirnas = screenMirnaList(shorescommon.readMirnaList('work/miRNAlists/4sumirnas.txt'), data)
  nn = generateTHEdata(data, mirnas)

  json = nn.to_json(orient='records')
  fp = open(output_dataset, 'w')
  fp.write(json)
  fp.close()
  print output_dataset
  # plotInterestingHairpinIds(data,output)

def generateTHEdata(data, mirnas):
  global minimum_entries, c_experiments
  new_data = []
  for mirna in mirnas:
    data_ = filterDataToMirna(data, mirna)
    for exp in c_experiments:
      data__ = data_[data_['exp'] == exp]
      if (data__.groupby('sample').__len__() != 7):
        #incomplete
        continue
      number = {}
      number['start'], number['end'] = len(data__[data__['position'] == 'start'].index), len(data__[data__['position'] == 'end'].index)
      data__ = shorescommon.fill4sutsWithZeros(data__)
      for pos in ['start', 'end']:
        data___ = data__[data__['position'] == pos]
        if number[pos] < minimum_entries:
          continue
        new_data.append({
          'mirna_id': mirna['id'],
          'class': mirna['class'],
          'position': pos,
          'exp': exp,
          'corr_p': data___['rpm'].corr(data___['mirnarpm']),
          'corr_s': data___['rpm'].corr(data___['mirnarpm'], method='spearman'),
          'sr_rpm': data___['rpm'].sum(),
          'mirna_rpm': data___.groupby('sample').first().reset_index()['mirnarpm'].sum()
          })
  return pd.DataFrame(new_data)

main()