"""
classify 4su /srds dataset into 3 experiments, rename samples.
"""

import numpy as np
import pandas as pd
from scipy import stats, integrate
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import pprint
import argparse
import code
import readline

def countRmiRNAs(data, field):
  counter = 0
  for matureId, group in data.groupby('matureId'):
    counter += group[field].unique()[0]
  return counter

def transformData(data):
  def classifier(samplenumber):
    samplenumber = int(samplenumber)
    if samplenumber >=26 and samplenumber <=32:
      return "EXP1"
    elif samplenumber >= 33 and samplenumber <=39:
      return "EXP2"
    elif samplenumber >= 40 and samplenumber <= 46:
      return "EXP3"
    else:
      return "undef"
  data['sample'] = data.apply(lambda row: row['sample'][-25:-23], axis=1)
  dict = []
  datagroupBysample = data.groupby('sample')
  for sampleInspected, datagroup in datagroupBysample:
    datagroupBysampleByhairpin = datagroup.groupby('hairpinId')
    for hairpinId, datagroupX in datagroupBysampleByhairpin:
      for prime, datagroupIIprime in datagroupX.groupby('prime'):
        datagroupBysampleByhairpinByPos = datagroupIIprime.groupby('position')
        for position, datagroupXX in datagroupBysampleByhairpinByPos:
          dict.append({
            'hairpinId': hairpinId,
            'sample': sampleInspected,
            'position': position,
            'prime': prime,
            'rpm': datagroupXX['rpm'].sum(),
            'mirnarpm': countRmiRNAs(datagroupXX, 'miRNAcount'),
            'exp': classifier(sampleInspected),
            'miRNAkind': datagroupXX['miRNAkind'].unique()[0]
            })
  return pd.DataFrame(dict)

def modifyName(name):
  return name.replace('.json', '_processed' + '.json')


parser = argparse.ArgumentParser()
parser.add_argument('dataset', help='dataset file')

args = parser.parse_args()
data = pd.read_json(open(args.dataset, 'r'))
data = transformData(data)
json = data.to_json(orient='records')
fp = open(modifyName(args.dataset), 'w')
fp.write(json)
fp.close()
print 'file written: ', modifyName(args.dataset)

